/******************************************************************************
*                                                                             *
*        **********                                                           *
*       ************                                                          *
*      ***        ***                                                         *
*     ***    ++    ***                                                        *
*     ***   +  +   ***                      CHIPCON                           *
*     ***   +                                                                 *
*     ***   +  +   ***                                                        *
*     ***    ++    ***                                                        *
*      ***        ***                                                         *
*       ************                                                          *
*        **********                                                           *
*                                                                             *
*******************************************************************************

Filename:     rf_test.c
Target:       cc2510
Author:       efu
Revised:      31/1-2006
Revision:     1.0

Description:
    This file provides 4 small tests which demonstrates use of the radio.

******************************************************************************/


#include "app_ex.h"
#include "cul.h"
#include "pingPong.h"


#define ADDRESS_0 0x01
#define ADDRESS_1 0x02

#define SEND     0
#define RECEIVE  1

#define SINGLE      0
#define CONTINUOUS  1
#define PING_PONG   2
#define PER_TEST    3
#define EXIT        4

#define RECEIVE_TIMEOUT               1200

#define PING_PONG_TIMEOUT             1200
#define PING_PONG_REQUEST             0x80
#define PING_PONG_RESPONSE            0x40

#define PER_RECEIVE_TIMEOUT          10000
#define PER_TOTAL_PACKET_NUMBER       1000
#define PER_TEST_REQUEST              0x20


void initRfTest(void);
void rf_test_main(void);
void receivePacket(UINT8 *receiveByte);
void sendPacket(UINT8 sendByte);
void printAddress(UINT8 address);
void printRSSI(INT8 rssi);
UINT32 getFrequency(void);
UINT8 getAddress(UINT8 a);


void receiveMode(void);
void singleMode(void);
void contionuousMode(void);
void pingPongMode(void);
void perTestMode(void);
void showState(void);
void conductPerTest(void);
void pingPongResponse(void);


//UINT8 address0; //my adresse
//UINT8 address1; //other adress
UINT8 RxTxState;
UINT8 myAddr;
UINT8 remoteAddr;
BOOL remoteAddressSet = FALSE;
BOOL myAddressSet = FALSE;


/******************************************************************************
* @fn  initRfTest
*
* @brief
*      Initializes components for the RF test application example.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void initRfTest(void)
{
   char *menuText[] = {(char *)" Sender?", (char *)" Receiver?", (char *)" Set Freq?", (char *)" Set MyAddr?", (char *)" Set RemoteAddr?"};
   BYTE result;
   UINT32 frequency = 2405000;


   initLcd();

   INIT_JOYSTICK();
   INIT_BUTTON();

   SET_MAIN_CLOCK_SOURCE(CRYSTAL);

   INIT_GLED();
   INIT_YLED();

   remoteAddressSet = FALSE;
   myAddressSet = FALSE;

   while((result = lcdMenu(menuText, 5)) > 1)
   {
      if(result == 2)
      {
         frequency = getFrequency();
      }
      else if(result == 3)
      {
         lcdUpdate((char *)"Set myAddress",(char *)"");
         myAddr = getAddress(myAddr);
         myAddressSet = TRUE;
      }
      else if(result == 4)
      {
         lcdUpdate((char *)"Set remoteAddr",(char *)"");
         remoteAddr = getAddress(remoteAddr);
         remoteAddressSet = TRUE;
      }
      else{ // ABORT_MENU
         return;
      }


   }

   if(result == 1) // Receiver
   {
      RxTxState = RECEIVE;
      if(myAddressSet == FALSE)
      {
         myAddr = ADDRESS_1;
      }
      if(remoteAddressSet == FALSE)
      {
         remoteAddr = ADDRESS_0;
      }
      lcdUpdate((char *)"Receiver",(char *)"");
   }
   else // Sender
   {
      RxTxState = SEND;
      if(myAddressSet == FALSE)
      {
         myAddr = ADDRESS_0;
      }
      if(remoteAddressSet == FALSE)
      {
         remoteAddr = ADDRESS_1;
      }
      lcdUpdate((char *)"Sender",(char *)"");
   }
   waitVisible(CLEARLY_VISIBLE);
   radioInit(frequency, myAddr);
}


/******************************************************************************
* @fn  rf_test_main
*
* @brief
*      Main function.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
#ifdef COMPLETE_APPLICATION
void rf_test_main(void){
#else
void main(void){
#endif
   UINT8 mode = 0;
   char *menuText[] = {(char *)" Single?", (char *)" Continuous?", (char *)" PingPong?", (char *)" PER Test?"};


   while(getJoystickDirection() != CENTRED);
   initRfTest();

   // Turning on interrupts
   INT_GLOBAL_ENABLE(INT_ON);


   while(!stopApplication())
   {

      // If set up as slave, the unit will only be in receive
      if(RxTxState == RECEIVE)
      {
         lcdUpdate((char *)"Receive",(char *)"Node");
         waitVisible(CLEARLY_VISIBLE);
         receiveMode();
         return;
      }
      else
      {
         mode = lcdMenu(menuText, 4);

         switch (mode){
         case SINGLE:
            lcdUpdate((char *)"Single mode", (char *)"");
            waitVisible(CLEARLY_VISIBLE);
            singleMode();
            break;

         case CONTINUOUS:
            lcdUpdate((char *)"Continuous Mode", (char *)"");
            waitVisible(CLEARLY_VISIBLE);
            contionuousMode();
            break;

         case PING_PONG:
            lcdUpdate((char *)"Ping Pong Mode", (char *)"");
            waitVisible(CLEARLY_VISIBLE);
            pingPongMode();
            break;

         case PER_TEST:
            lcdUpdate((char *)"PER Mode", (char *)"");
            waitVisible(CLEARLY_VISIBLE);
            perTestMode();
            break;

         case ABORT_MENU:
            return;
         }
      }
      while(getJoystickDirection() != CENTRED);
      GLED = YLED = LED_OFF;
   }

   STOP_RADIO();
   return;
}


/******************************************************************************
* @fn  singleMode
*
* @brief
*      Function for transferring one packet
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void singleMode(void)
{
   char *singleMenu[] = {(char *)" Send?"};
   BYTE sendBuffer[] = "Hello";
   BOOL res;


   while(TRUE)
   {
      if(lcdMenu(singleMenu, 1) == 0)
      {
         YLED = LED_ON;
         res = radioSend(sendBuffer, sizeof(sendBuffer), remoteAddr, DO_ACK);
         YLED = LED_OFF;
         if(res == TRUE)
         {
            GLED = LED_ON;
            lcdUpdateLine(LINE1, (char *)"OK!");

         }
         else
         {
            GLED = LED_OFF;
            lcdUpdateLine(LINE1, (char *)"Failed...");
         }
         waitVisible(CLEARLY_VISIBLE);
         GLED = LED_OFF;
      }
      else
      {   // exit is selected...
         return;
      }
   }
}


/******************************************************************************
* @fn  contionuousMode
*
* @brief
*      Function for transferring packets until stopped by user.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void contionuousMode(void)
{
   BOOL res;
   BYTE sendBuffer[] = "Hello";
   showState();

   while(!stopApplication()){
      GLED = LED_OFF;
      YLED = LED_ON;
      res = radioSend(sendBuffer, sizeof(sendBuffer), remoteAddr, DO_ACK);
      YLED = LED_OFF;

      if(res == TRUE)
      {
         GLED = LED_ON;
         lcdUpdateLine(LINE1, (char *)"OK!");
         //printRSSI();
      }
      else
      {
         GLED = LED_OFF;
         lcdUpdateLine(LINE1, (char *)"Failed...");
      }
      waitVisible(CLEARLY_VISIBLE);
   }
}


/******************************************************************************
* @fn  pingPongMode
*
* @brief
*      Function for sending packets between two nodes.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void pingPongMode(void)
{
    BOOL res;
    BYTE sendBuffer = PING_PONG_REQUEST;
    BYTE* buffer;
    BYTE length;
    BYTE sender;


    //showState();

    pingPongInit();


   while( !stopApplication() ){
      YLED = LED_ON;
      res = radioSend(&sendBuffer, 1, remoteAddr, DO_ACK);
      sendBall(ORIENTED_RIGHT);
      YLED = LED_OFF;
      if(res == TRUE)
      {
         res = radioReceive(&buffer, &length, PING_PONG_TIMEOUT,&sender);
      }
      else
      {
         res = FALSE;
      }


      if(res == TRUE)
      {
         receiveBall(ORIENTED_RIGHT);
         GLED = LED_ON;
         // lcdUpdateLine(LINE1, "OK!");
         lcdUpdate((char *)"OK!", (char *)"");
         printRSSI(buffer[length]);
      }
      else
      {
         GLED = LED_OFF;
         // lcdUpdateLine(LINE1, "Failed...");
         lcdUpdate((char *)"Failed...", (char *)"");
         waitVisible(CLEARLY_VISIBLE);
      }
      waitVisible(CLEARLY_VISIBLE);
      GLED = LED_OFF;
   }

}


/******************************************************************************
* @fn  perTestMode
*
* @brief
*      Function for performing a packet error rate (PER) test.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void perTestMode(void)
{
   BYTE sendBuffer = PER_TEST_REQUEST;
   char number[16];

   BYTE* receiveBuffer;
   BYTE length;
   BYTE res;
   BYTE sender;
   WORD lastPacketNumber = 0;
   WORD thisPacketNumber;
   WORD failed = 0;


   YLED = LED_ON;
   res = radioSend(&sendBuffer, 1, remoteAddr, DO_ACK);
   YLED = LED_OFF;

   if(res == TRUE)
   {
      while(lastPacketNumber < PER_TOTAL_PACKET_NUMBER)
      {
         res = radioReceive(&receiveBuffer, &length, PER_RECEIVE_TIMEOUT, &sender);
         if(res == TRUE)
         {
            thisPacketNumber = *((WORD*) receiveBuffer);
            failed += (thisPacketNumber - lastPacketNumber - 1);
            lastPacketNumber = thisPacketNumber;
            GLED = LED_ON;
         }
         else
         {
            GLED = LED_OFF;
            failed += (PER_TOTAL_PACKET_NUMBER - lastPacketNumber);
            lastPacketNumber = PER_TOTAL_PACKET_NUMBER;
         }
      }
      if((failed == 0) &&(lastPacketNumber != 0))
      {
         lcdUpdateLine(LINE2,(char *)"Correct!");
      }
      failed = failed / 10;
      sprintf(number, (char *)"PER: %d", failed);
      lcdUpdateLine(LINE1,number);
      lcdUpdateChar(LINE1, 8, '%');
   }
   else
   {
      lcdUpdate((char *)"Node Unreachable",(char *)"");
   }

   haltApplicationWithLED();

   return;
}


/******************************************************************************
* @fn  receiveMode
*
* @brief
*      Function for receiving data.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void receiveMode(void)
{
   BYTE* receiveBuffer;
   BYTE length;
   BYTE res;
   BYTE sender;

   pingPongInit();

   while( !stopApplication() )
   {
      res = radioReceive(&receiveBuffer, &length, RECEIVE_TIMEOUT, &sender);

      if(res == TRUE)
      {
         GLED = LED_ON;
         if(*receiveBuffer == PING_PONG_REQUEST)
         {
            pingPongResponse();
         }
         else if(*receiveBuffer == PER_TEST_REQUEST)
         {
            conductPerTest();
         }
         else
         {
            printRSSI(receiveBuffer[length]);
            showState();
         }
      }
      else
      {
         lcdUpdate((char *)"Waiting",(char *)"");
         GLED = LED_OFF;
      }

      GLED = LED_OFF;
   }
   return;
}


/******************************************************************************
* @fn  showState
*
* @brief
*      Print "Sending" or "Receiving" on LINE1.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void showState(void)
{
   if (RxTxState == SEND){
      lcdUpdateLine(LINE1, (char *)"Sending");
   }
   else{
      lcdUpdateLine(LINE1, (char *)"Receiving");
   }
   halWait(0xFF);
}


/******************************************************************************
* @fn  pingPongResponse
*
* @brief
*      Handles response to pingpong.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void pingPongResponse(void)
{
  BYTE transmitBuffer;

  receiveBall(ORIENTED_LEFT);
  halWait(1);
  YLED = LED_ON;
  transmitBuffer = PING_PONG_RESPONSE;
  radioSend(&transmitBuffer, 1, remoteAddr, DO_NOT_ACK);
  sendBall(ORIENTED_LEFT);
  YLED = LED_OFF;
}


/******************************************************************************
* @fn  conductPerTest
*
* @brief
*      Sends 1000 packets to calculate PER.
*
* Parameters:
*
* @param  void
*
* @return void
*
******************************************************************************/
void conductPerTest(void)
{
    WORD i;
    WORD sendBuffer[2];

    lcdUpdate((char *)"Conducting", (char *)"PER test.");

    sendBuffer[1] = PER_TOTAL_PACKET_NUMBER;

    for(i=1; i < PER_TOTAL_PACKET_NUMBER+1 ; i++)
    {
        halWait(8);
        sendBuffer[0] = i;
        radioSend((BYTE*) sendBuffer, sizeof(sendBuffer), remoteAddr, DO_NOT_ACK);
    }
}


/******************************************************************************
* @fn  printRSSI
*
* @brief
*      Printing RSSI on the LCD-display.
*
* Parameters:
*
* @param  INT8 rssi
*         New RSSI value.
*
*
* @return void
*
******************************************************************************/
void printRSSI(INT8 rssi)
{
   char rssiString[16];
   UINT8 rssiVal = 0;
   rssiVal = (UINT16) rssi;
   if(rssiVal >= 128)
   {
      rssiVal = (((rssiVal - 256)/2) -72);
   }
   else
   {
      rssiVal = ((rssiVal/2) -72);
   }

   sprintf(rssiString,  (char const *)"%d dBm", (INT8)rssiVal);

   lcdUpdateLine(LINE2, rssiString);
}


/******************************************************************************
* @fn  getFrequency
*
* @brief
*      Get current frequency.
*
* Parameters:
*
* @param  void
*
* @return UINT32
*         Current frequency.
*
******************************************************************************/
UINT32 getFrequency(void)
{
   UINT8 selection;
   char *menuText[] = {(char*)" 2405", (char*)" 2410", (char*)" 2415", (char*)" 2420", (char*)" 2425", (char*)" 2430", (char*)" 2435", (char*)" 2440", (char*)" 2445", (char*)" 2450", (char*)" 2455", (char*)" 2460", (char*)" 2465", (char*)" 2470", (char*)" 2475", (char*)" 2480", (char*)" 2485"};
   selection = lcdMenu(menuText, 17);
   if(selection != ABORT_MENU)
   {
      return (2405000 + 5000*selection);
   }
   else
   {
      return (2405000);
   }

}


/******************************************************************************
* @fn  getAddress
*
* @brief
*      Get current address.
*
* Parameters:
*
* @param  UINT8 a
*
* @return UINT8
*
******************************************************************************/
UINT8 getAddress(UINT8 a)
{
   UINT8 address = a;
   char line[16];
   JOYSTICK_DIRECTION jsv;

   while(getJoystickDirection() != CENTRED);

   while((jsv = getJoystickDirection()) != RIGHT)
   {
      while(getJoystickDirection() != CENTRED);

      if(jsv == UP)
      {
         address++;
      }
      else if (jsv == DOWN)
      {
         address--;
      }

      sprintf(line,(char *)"Address: %d     ",address);
      lcdUpdateLine(LINE2, line);
   }
   return address;
}


/******************************************************************************
* @fn  clockmodes_init
*
* @brief
*     Setup for incorperating the functions in this
*     module with the menu
*
* Parameters:
*
* @param  APPLICATION *a
*         Main application
*
* @return void
*
******************************************************************************/
#ifdef COMPLETE_APPLICATION
void rf_test_init(APPLICATION *a)
{
   a->menuText = (char *)"RF Test";
   a->description = (char *)"Radio comm.";
   a->main_func = rf_test_main;
}
#endif


